//! Handles arg parsing
use super::*;
use std::fmt;

/// The name of the process that was invoked
#[inline] pub fn prog_name() -> &'static str
{
    lazy_static! {
	static ref NAME: String = std::env::args().next().unwrap();
    }
    &NAME[..]
}

#[inline] fn options() -> impl Iterator<Item = impl fmt::Display>
{
    struct Opt(&'static [&'static str], Option<&'static str>, &'static str, usize, bool);

    impl Opt
    {
	const fn with_def(self, vl: bool) -> Self
	{
	    Self(self.0, self.1, self.2, self.3, vl)
	}
    }

    macro_rules! opt {
	($($name:literal),* => $num:literal; $desc:literal $(, $def:literal)?) => {
	    {
		let o = Opt(&[$($name),*], Some($num), $desc, 1, false);
		$(let o = o.with_def($def);)?
		    o
	    }
	};
	($($name:literal),* => $num:literal; * $tabs:literal $desc:literal $(, $def:literal)?) => {
	    {
		let o = Opt(&[$($name),*], Some($num), $desc, $tabs, false);
		$(let o = o.with_def($def);)?
		    o
	    }
	};
	($($name:literal),*; $desc:literal $(, $def:literal)?) => {
	    {
		let o = Opt(&[$($name),*], None, $desc, 1, false);
		$(let o = o.with_def($def);)?
		    o
	    }
	};
	($($name:literal),*; * $tabs:literal  $desc:literal  $(, $def:literal)?) => {
	    {
		let o = Opt(&[$($name),*], None, $desc, $tabs, false);
		$(let o = o.with_def($def);)?
		    o
	    }
	};
    }
    impl fmt::Display for Opt
    {
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result
	{
	    write!(f, " {}", self.0.iter().join(", "))?;
	    if let Some(mx) = self.1 {
		write!(f, "  {}", mx)?;
	    }
	    write!(f, ":")?;
	    for ch in std::iter::repeat('\t').take(self.3)
	    {
		use std::fmt::Write;
		f.write_char(ch)?;
	    }
	    
	    write!(f,"{}", self.2)?;
	    if self.4 {
		use recolored::Colorize;
		write!(f, " ({})", "default".bold().bright_red())?;
	    }
	    Ok(())
	}
    }

    iter![
	opt!("--limit", "-l" => "<max children>"; "Limit the number of child processes running at once."),
	opt!("--auto-limit", "-m"; * 2 "Limit to max number of CPUs + 1", true),
	opt!("--unlimit", "-U"; * 3 "No limit to number of processes spawned at once")
    ]
}

/// Print usage
pub fn print_usage()
{
    use recolored::Colorize;
    
    println!(r#"batchtask v{version} - advanced process batching
 by {author} with <3 (licensed GPL3+)"#,
	     version = env!("CARGO_PKG_VERSION"),
	     author = env!("CARGO_PKG_AUTHORS"));
    println!(r#"
Usage: {prog} [OPTIONS]...
Usage: {prog} --help

[{OPTIONS}]:
{opts}
"#,
	     prog = prog_name(),
	     OPTIONS = "OPTIONS".bright_blue().bold(),
	     opts = options().map(|x| lazy_format!("{}", x)).join("\n"));
}

/// Print usage then exit
#[inline(always)] pub fn usage() -> !
{
    print_usage();
    std::process::exit(1)
}
